<?php

namespace Drupal\search_api_solr_nlp\EventSubscriber;

use Drupal\search_api_solr\Event\PostConfigFilesGenerationEvent;
use Drupal\search_api_solr\Event\PostConfigSetGenerationEvent;
use Drupal\search_api_solr\Event\SearchApiSolrEvents;
use SolariumNlp\Nlp;
use Symfony\Component\EventDispatcher\EventSubscriberInterface;

/**
 * Search API events subscriber.
 */
class SearchApiSolrSubscriber implements EventSubscriberInterface {

  /**
   * @var array The models.
   */
  protected static $models = [];

  /**
   * Alter the newly assembled Solr configuration files.
   *
   * @param \Drupal\search_api_solr\Event\PostConfigFilesGenerationEvent $event
   *   The event.
   */
  public function onPostConfigFilesGeneration(PostConfigFilesGenerationEvent $event): void {
    $files = $event->getConfigFiles();
    $models = [];
    if (isset($files['schema_extra_types.xml'])) {
      if (preg_match_all('@Model="([^"]+\.bin)"@', $files['schema_extra_types.xml'], $matches)) {
        foreach($matches[1] as $match) {
          $models[] = $match;
        }
      }
    }
    self::$models = array_unique($models);
  }

  /**
   * Alter the zip archive of newly assembled Solr configuration files.
   *
   * @param \Drupal\search_api_solr\Event\PostConfigSetGenerationEvent $event
   *   The event.
   *
   * @throws \ZipStream\Exception\FileNotFoundException
   * @throws \ZipStream\Exception\FileNotReadableException
   */
  public function onPostConfigSetGeneration(PostConfigSetGenerationEvent $event): void {
    $zip = $event->getZipStream();
    $nlp = new Nlp();
    foreach(self::$models as $model) {
      $zip->addFileFromPath($model, $nlp->getOpenNlpDemoModelPath($model));
    }
  }

  /**
   * {@inheritdoc}
   */
  public static function getSubscribedEvents(): array {
    // Workaround to avoid a fatal error during site install in some cases.
    // @see https://www.drupal.org/project/facets/issues/3199156
    if (!class_exists('\Drupal\search_api_solr\Event\SearchApiSolrEvents', TRUE)) {
      return [];
    }

    return [
      SearchApiSolrEvents::POST_CONFIG_FILES_GENERATION => 'onPostConfigFilesGeneration',
      SearchApiSolrEvents::POST_CONFIG_SET_GENERATION =>  'onPostConfigSetGeneration',
    ];
  }
}
